package com.jdbernard.teammaker

import groovy.beans.Bindable
import groovy.swing.SwingBuilder
import java.awt.GridBagConstraints as GBC
import java.awt.GridBagLayout
import java.awt.BorderLayout
import javax.swing.DefaultListModel
import javax.swing.JFrame
import javax.swing.JOptionPane
import org.apache.log4j.Logger

public class TeamMaker {

    /* ======== MODEL ======== */

    public static def swing = new SwingBuilder()
    public static final String version = "0.3"

    def frame
    def team1List
    def team2List
    def team1WinsButton
    def team2WinsButton
    def newGameButton
    def sittingList

    protected Logger log = Logger.getLogger(getClass())

    PlayerChooser playerChooser = new WeightedChooser(hardLimit: 2)
    OddsCalculator oddsCalculator = new OddsCalculator()

    class Observables {
        @Bindable boolean inGame = false
        @Bindable int teamSize = 4
    }
    def model = new Observables()



    /* ======== VIEW ======== */

        /*
           +--------------------------------------------------+
           | JDB TeamMaker v0.2                          _[]X |
           +--------------------------------------------------+
           |                                                  |
           | +-Current-Game--------------+ +-Sitting-Players+ |
           | |                           | |                | |
           | | Team 1          Team 2    | | Jeptha       3 | |
           | |   Derrick         Eric    | | Jack         2 | |
           | |   Shane           Brandon | | Sam          2 | |
           | |   Shane           Darren  | | Greg         2 | |
           | |   Christian       Brett   | | Frank        1 | |
           | +---------------------------+ | Chris D      1 | |
           |                               | Joshua       1 | |
           |                               |                | |
           |                               |                | |
           |                               |                | |
           |                               |                | |
           |                               |                | |
           |                               +----------------+ |
           +--------------------------------------------------+

         */

    private void init() {
        frame = swing.frame(title: "JDB TeamMaker v$version",
            size: [600, 400], locationRelativeTo: null,
            defaultCloseOperation: JFrame.EXIT_ON_CLOSE) {

            panel() {
                gridBagLayout()

                panel(constraints: gbc(gridx: 0, gridy: 0, fill: GBC.BOTH,
                        insets: [5, 5, 5, 0], weightx: 3, weighty: 1),
                    border: titledBorder(title: 'Current Game')) {
                    
                    gridBagLayout()

                    label('Team A', constraints: gbc(gridx: 0, gridy: 0,
                        fill: GBC.BOTH, insets: [5, 5, 0, 0]))

                    label('Team B', constraints: gbc(gridx: 1, gridy: 0, 
                        fill: GBC.BOTH, insets: [5, 5, 0, 5]))
                    
                    def teamListRenderer = new PlayerListCellRenderer(this,
                        showStats: false, colored: false)

                    team1List = list(cellRenderer: teamListRenderer,
                        constraints: gbc(gridx: 0, gridy: 1, fill: GBC.BOTH,
                        insets: [5, 5, 0, 0], weightx: 2, weighty: 2),
                        model: new DefaultListModel())

                    team1WinsButton = button('Team A Wins',
                        constraints: gbc(gridx: 0, gridy: 2, anchor:
                            GBC.CENTER, insets: [5, 5, 5, 0]),
                        enabled: bind { model.inGame },
                        actionPerformed: { declareWinner(team1List)})

                    team2List = list(cellRenderer: teamListRenderer,
                        constraints: gbc(gridx: 1, gridy: 1, fill: GBC.BOTH,
                        insets: [5, 5, 0, 5], weightx: 2, weighty: 2),
                        model: new DefaultListModel())

                    team2WinsButton = button('Team B Wins',
                        constraints: gbc(gridx: 1, gridy: 2,
                            anchor: GBC.CENTER, insets: [5, 5, 5, 5]),
                        enabled: bind { model.inGame },
                        actionPerformed: { declareWinner(team2List)} )

                }

                scrollPane(constraints: gbc(gridx: 1, gridy: 0, gridwidth: 2,
                        fill: GBC.BOTH, insets: [5, 5, 0, 5], weighty: 1),
                    border: titledBorder(title: 'Sitting Players')) {

                    sittingList = list(cellRenderer: new PlayerListCellRenderer(
                            this, showStats: true, colored: true),
                            model: new SortedPlayerModel())
                }

                button('Next Game', constraints: gbc(gridx: 0, gridy: 1,
                    anchor: GBC.CENTER, insets: [5, 5, 5, 0]),
                    enabled: bind { !model.inGame},
                    actionPerformed: { newGame() })

                button('Add Player', constraints: gbc(gridx: 1, gridy: 1,
                    anchor: GBC.CENTER, insets: [5, 5, 5, 0]),
                    actionPerformed: { addPlayer() })

                newGameButton = button('Delete Player',
                    constraints: gbc(gridx: 2, gridy: 1, anchor: GBC.CENTER,
                        insets: [5, 5, 5, 5]),
                    actionPerformed: {
                        sittingList.model.removeElement(
                            sittingList.selectedValue) })

            }
        }

    }


    

    /* ======== CONTROLLER ======== */

    public static void main(String[] args) {
        def inst = new TeamMaker()
        inst.frame.show()
    }

    TeamMaker() {
        log.fatal("Is this thing on?")
        init()
    }

    public int getSpotsOpen() {
        def spots =  (model.teamSize - team1List.model.size()) +
                     (model.teamSize - team2List.model.size())
        return (spots == 0 ? model.teamSize : spots)
    }

    private void addPlayer() {
        def name = JOptionPane.showInputDialog(frame,
            "Enter the new player's name: ", "New Player...",
            JOptionPane.QUESTION_MESSAGE)

        if (!name) return

        def player = new Player()
        player.name = name
        player.gamesSat = 0
        sittingList.model.addElement(player)

        // recalculate odds
        def players = sittingList.model.collect { it }
        swing.doOutside {
            oddsCalculator.recalculate(players, getSpotsOpen(), playerChooser)
            swing.edt { sittingList.repaint() }
        }
    }

    private void newGame() {
        if (team1List.model.size() < model.teamSize)
            populate(team1List)

        if (team2List.model.size() < model.teamSize)
            populate(team2List)

        model.inGame = true

        // recalculate odds
        def players = sittingList.model.collect { it }
        swing.doOutside {
            oddsCalculator.recalculate(players, getSpotsOpen(), playerChooser)
            swing.edt { sittingList.repaint() }
        }
    }

    private void declareWinner(def teamList) {
        sittingList.model.each { it.gamesSat++ }
        teamList.model.each { player ->
            player.gamesSat = 0
            sittingList.model.addElement(player)
        }

        //recalculate odds
        def players = sittingList.model.collect { it }
        swing.doOutside {
            oddsCalculator.recalculate(players, getSpotsOpen(), playerChooser)
            swing.edt { sittingList.repaint() }
        }

        teamList.model.clear()
        teamList.repaint()
        sittingList.repaint()
        model.inGame = false
    }

    private void populate(def teamList) {
        while (teamList.model.size() < model.teamSize) {
            def player = playerChooser.choose(sittingList.model)
            teamList.model.addElement(player)
            sittingList.model.removeElement(player)
        }
    }

}
